Module: Aikido::Zen::DetachedAgent

Defined in:
lib/aikido/zen/detached_agent/front_object.rb,
lib/aikido/zen/detached_agent/agent.rb,
lib/aikido/zen/detached_agent/server.rb

Overview

dRB Front object that will work as a bridge communication between child & parent processes. Every method is called from the child but it runs in the parent process.

Defined Under Namespace

Classes: Agent, FrontObject, Server